<?php

require 'assets/php/models/Room.php';

class TimeSlot
{
    protected DateTime $start;
    protected DateTime $end;
    protected ?Room $room = null;

    public function __set(string $name, mixed $value): void
    {
        if (is_null($this->room))
            $this->room = new Room();

        switch ($name) {
            case 'id':
                $this->room->id = $value;
                break;

            case 'type':
                $this->room->type = $value;
                break;

            case 'location':
                $this->room->location = $value;
                break;

            case 'start_time':
                $this->start = new DateTime($value);
                break;

            case 'end_time':
                $this->end = new DateTime($value);
                break;

            case 'room':
                $this->room = $value;
                break;

            default:
                break;
        }
    }

    public function __get($name): mixed
    {
        if (property_exists($this, $name))
            return $this->$name;
    }

    public function __clone()
    {
        $this->start = clone $this->start;
        $this->end = clone $this->end;
    }

    public function display(): void
    {
        $room_name = $this->room->id;
        $location = $this->room->location;
        $type = $this->room->type;
        $start_time = $this->start->format('H:i');
        $end_time = $this->end->format('H:i');

        include("assets/php/result.php");
    }

    public function cut(TimeSlot $slot): array
    {
        $cutout = [];

        $ss_diff = $this->start->diff($slot->start);
        $se_diff = $this->start->diff($slot->end);
        $es_diff = $this->end->diff($slot->start);
        $ee_diff = $this->end->diff($slot->end);

        // If $slot starts and ends before $this starts and ends respectively
        if ($ss_diff->invert && $ee_diff->invert) {
            $clone = clone $this;
            $clone->start->add($se_diff);
            array_push($cutout, $clone);
        }
        // If $slot starts and ends after $this starts and ends respectively
        elseif (!$ss_diff->invert && !$ee_diff->invert) {
            $clone = clone $this;
            $clone->end->add($es_diff);
            array_push($cutout, $clone);
        }
        // If $slot splits this in two
        elseif (!$ss_diff->invert && $ee_diff->invert) {
            $pre = clone $this;
            $post = clone $this;
            $pre->end->add($es_diff);
            $post->start->add($se_diff);
            array_push($cutout, $pre, $post);
        } elseif (!($ss_diff->invert && !$ee_diff->invert)) {
            array_push($cutout, clone $this);
        }

        return array_filter($cutout, function (TimeSlot $e) {
            return $e->start != $e->end;
        });
    }
}
